// Copyright (C) 1996 by Wolfgang Platzer
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

package cryptiX.user;

import java.io.*;
import cryptiX.asn1.*;
import cryptiX.rsa.*;
import cryptiX.crypto.Md5Hash;
import cryptiX.utils.*;

/**
 * This class represents a certificate request from Netscape Navigator 3.0
 */

public class NetscapeCertRequest
{
    private ASN1 spkac = null;          // SignedPublicKeyAndChallange ASN1 structure

    private PublicKey publicKey = null; // the public inside this structure

    /**
     * create a spkac from a PEM or DER file
     */
    public NetscapeCertRequest(String fileName) throws IOException, CodingException {
        spkac = new ASN1(fileName);
                        // create a PublicKey defined by SubjectPublicKeyInfo
        publicKey = PublicKey.valueOf(new ASN1(spkac.getComponentAt(0)).getComponentAt(0));
    }

    /**
     * create a spkac form a PEM or DER array
     */
    public NetscapeCertRequest(byte[] arr) throws CodingException {
        spkac = new ASN1(arr);
                        // create a PublicKey defined by SubjectPublicKeyInfo
        publicKey = PublicKey.valueOf(new ASN1(spkac.getComponentAt(0)).getComponentAt(0));
    }

    /**
     * verifies the self signed public key
     */
    public boolean verify() throws CodingException {
                        // get the PublicKeyAndChallange Object (position 1)
        ASN1 pkac = new ASN1(spkac.getComponentAt(0));

                        // get the signatureAlgorithm from spkac (position 2)
        String signatureAlgorithm = Util.AlgIDAlgorithm(spkac.getComponentAt(1));

                        // get the signature from spkac (position 3)
        ASN1Object sigObject = spkac.getComponentAt(2);
        byte[] signature = (byte[])(sigObject).getValue();

                        // check the signature with appropriate algorithm
        return publicKey.checkSignature(pkac.toByteArrayDER(), signature, signatureAlgorithm);
    }

    /**
     * return the public key from this spkac structure
     */
    public PublicKey getPublicKey() {
        return publicKey;
    }
}

