//
// Loader.java --
//
// Author: Pekka Nikander <pnr@tequila.nixu.fi>
//
// Copyright 1996 by Nixu Oy, Helsinki, Finland
//
// Created: Fri Mar 29 09:46:19 1996
// Last modified: Sun Mar 31 22:54:54 1996 pnr
//

package cryptiX.utils;

import  java.lang.ClassLoader;

/**
 * The Loader package can be used to dynamicly load java classes
 * to the execution environment.  In fact, the only purpose of this
 * class is to in a way circumwent the standard ClassLoader class protection
 * by providing and interface for direct, runtime controlled class
 * loading.  If the ClassLoader.loadClass() method were public,
 * this class would be mostly useless.
 *
 * This class is most usable with dynamic factories.  The
 * Factory class provides an abstract superclass for dynamic
 * factories.
 *
 * @see cryptiX.utils.Factory
 *
 */

/*
 * NOTE.  I tried to do this by subclassing ClassLoader,
 * 	  but that didn't work.  However, I still think
 *	  that this is a quite reasonable encapsulation anyway.
 */

public class Loader  {
  /**
   * Prevent instantiation
   */
  private Loader() {}

  /**
   * Loads and resolves a class with the given name.
   * Uses whatever classloader is used to load this particular class.
   * @param name fully qualified name (including package) of the class to load.
   * @return the class loaded, or null if an error occured.
   */
  public static Class loadClass(String name) {
    try {
      return Class.forName(name);
    } catch (ClassNotFoundException e) {
      System.err.println("Loader: class loading failed for class " + name);
      System.err.println("Loader: " + e.getMessage());
      return null;
    }
  }
}
